# -*- coding: utf-8 -*-

"""Package settings."""

__title__ = 'lnk'
__version__ = '0.1.4'
__author__ = 'Peter Goldsborough'
__email__ = 'petergoldsborough@hotmail.com'
__license__ = 'MIT'
__copyright__ = 'Copyright 2015 Peter Goldsborough'
__url__ = 'https://github.com/goldsborough/lnk'
